#/usr/bin/python
# -*- coding: utf-8 -*-

from meta import GeometricalObject
from pointsOfIntersection import PointsOfIntersection
from ..utilities import normalize

from scipy import cross, dot
from scipy.linalg import norm as vabs

class Sphere(GeometricalObject):
	def __init__(self, pos, radius, dir = None, scale = None):
		GeometricalObject.__init__(self, pos, dir, scale)
		self.radius = radius
	
	def intersectionWithRay(self, ray):
		points = PointsOfIntersection(ray)
		
		start = ray.pos-self.pos
		
		a = dot(ray.dir, ray.dir)
		b = dot(ray.dir, start) * 2.0
		c = dot(start, start) - self.radius * self.radius
		
		discriminant = b * b - 4.0 * a * c
		
		if discriminant > 0:
			q = discriminant**.5
			
			t = (-b + q) / (2.0 * a)
			points.addPoint(dist = t, norm = ray.getPoint(t)-self.pos)
			
			t = (-b - q) / (2.0 * a)
			points.addPoint(dist = t, norm = ray.getPoint(t)-self.pos)
		
		elif discriminant == 0:
			t = -b / (2.0 * a)
			points.addPoint(dist = t, norm = ray.getPoint(t)-self.pos)
		
		return points
	
	def pointIsPartOf(self, point):
		if vabs(point - self.pos) <= self.radius:
			return True
		else:
			return False
